/*Reaktiivinen ohjelmointi tarkoittaa sellaisten sovellusten kehittämistä jotka
perustuvat jatkuvan datavirran (streamin) reaaliaikaiseen syöttämiseen (push) sovellukseen.
Tällaista voi olla esim. Web-API:sta tuleva jatkuvasti muuttuva tieto, IoT-laitteesta
tuleva tieto, tai käyttäjän antama jatkuva syöte. Dataa emitoidaan streamista sovellukseen
yksi dataelementti kerrallaan.

Angular tukee reaktiivista ohjelmointia, sillä siinä on sisäänrakennettuna Rxjs -kirjasto
(ja signaalit, joita tässä esimerkissäei kuitenkaan käytetä).

Observable (tarkkailtava) on olio joka lähettää streamin. Observer (tarkkailija) on olio
joka vastaanottaa ja käsittelee streamin. Streami tilataan subscribe() -metodilla
(cold observable). Streami voi tulla myös ilman tilausta (hot Observable).

Tässä esimerkissä streami tulee käyttäjän syöttestä eli inputista.  Esimerkin toiminta vaatii
ReactiveFormsModulen importtauksen.
*/

import { Component } from '@angular/core';
import { FormControl, ReactiveFormsModule} from '@angular/forms';
import { debounceTime } from 'rxjs/operators';

@Component({
  selector: 'app-observable1',
  imports: [ReactiveFormsModule],
  template: `
       <h2>Observable1</h2>
       <p>Reaktiivinen stream tulee kentän alle ja konsoliin kun kirjoitat merkkejä<br /> tekstikenttään.
       Data on sovelluksessa arvottua 'feikkidataa'.</p>
      <input type="text" placeholder="Kirjoita paikan nimi" [formControl]="searchInput">
      <p>{{tulostus}}</p>
    `
})
export class Observable1Component {

/*
Formcontrol -olio on reaktiivisen lomakkeen perusrakenneosa.
searchInput -niminen olio on bindattu yllä olevaan lomakkeeseen.
*/
  searchInput: FormControl = new FormControl('');
  tulostus: string = '';

  constructor() {
      // valueChanges on property joka tekee searchInputista observablen joka lähettää eli emitoi
      // datastreamin eventtejä aina kun lomakkeella oleva arvo vaihtuu eli value changes
      this.searchInput.valueChanges
      .pipe(debounceTime(500)) // debouncetime aiheuttaa viiveen emitoitavien eventtien välissä
      .subscribe(data => this.getTemperatureFromServer(data)); // tilataan (subscribe) Observable stream
  }

  getTemperatureFromServer(place: string) {
    const max = 30;
    const min = 0;
    const drawed: any = (Math.random() * (max - min) + min).toFixed(1); // arvotaan lukuarvo lämpötilalle

    // estetään error jos kenttä tyhjennetään
    if (place === '') {
      place = 'Tyhjälä';
    }

    console.log(`Lämpötila paikassa ${place} on ${drawed} astetta.`);
    this.tulostus = `Lämpötila paikassa ${place} on ${drawed} astetta.`;
  }
}
