/*
Tämä esimerkki eroaa edellisestä reaktiivisesta esimerkistä siten että streameja
on kaksi: 1) käyttäjän syöte 2) Web-APIsta tuleva data.

SwitchMap -metodi vaihtaa streamin lähteen käyttäjän syötteestä Web-APIin.
Kun käyttäjä syöttaa kirjaimia input-kenttään, tilataan (subscribe) aina uusi
event stream (http-pyyntö). jos käyttäjä ehtii syöttää uuden merkin ennen kuin
tilattu stream on tullut Web-APIsta, niin käyttäjän edellisen syötteen streami peruutetaan.
Tämä tehostaa sovelluksen toimintaa koska turhat streamaukset jäävät pois.
SwitchMap-metodi mahdollistaa streamien peruutuksen.

*/
import { Component } from '@angular/core';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { switchMap, debounceTime, distinctUntilChanged, retry } from 'rxjs/operators';

@Component({
    selector: 'app-observable2',
    imports: [ReactiveFormsModule],
    template: `
      <h2>Observable2</h2>
      <p>Haetaan <a href="https://openweathermap.org/" target="_blank">OpenWeatherMapin</a> API-palvelusta säätietoja reaktiivisesti</p>
      <p>Streamille on asetettu sekunnin viive (debouncetime)</p>
      <p>Seuraa datan tuloa myös selaimen konsolista. Jos haettua paikkakuntaa ei ole,<br />
      tulee virhe ja yritetään hakea uudelleen niin kauan että käyttäjä antaa paikkakunnan joka löytyy.</p>
      <p>Kokeile vaikkapa "kem", "kemi", "kemijärvi"</p>

      <input type="text" placeholder="Paikkakunta" [formControl]="searchInput">
      <h3>{{weatherdata}}</h3>
    `
})

export class Observable2Component {
    private baseWeatherURL = 'http://api.openweathermap.org/data/2.5/weather?q=';
    private urlSuffix = '&appid=ca3f6d6ca3973a518834983d0b318f73&units=metric';

    /*
    Formcontrol -olio on reaktiivisen lomakkeen perusrakenneosa.
    searchInput -niminen olio on bindattu yllä olevaan lomakkeeseen.
    */
    searchInput: FormControl = new FormControl();
    weatherdata: string = '';

    constructor(private http: HttpClient) {
        // valueChanges tuottaa observablen
        this.searchInput.valueChanges
            .pipe(debounceTime(1000), // debouncetime aiheuttaa viiveen emitoitavien eventtien välissä
                // switchMap vaihtaa streamin lähteeksi toisen Observablen joka on this.getWeather()
                // city otetaan parametrina ekalta Observablelta
                distinctUntilChanged(), // jos sama hakutermi toistuu, se ignorataan
                switchMap(city => this.getWeather(city)),
                // yrittää uudelleen jos paikkakuntaa ei löydy. Tekee turhia Api-kutsuja.
                // Homman voisi hoitaa hieman tehokkaammin esim. oletusObservablella jos serveriltä tulee 404
                retry())
            .subscribe({
                next:
                    res => {
                        // data tulee sääpalvelusta observable-oliosta jonka main-propertyssä ovat halutut tiedot
                        console.log(typeof res); // tulostetaan responsen tyyppi konsoliin
                        console.log(res); // tulostetaan koko response konsoliin
                        console.log(res.main); // tulostetaan koko responsen main -property konsoliin
                        if (!res.main) {
                            this.weatherdata = 'Dataa ei löydy';
                        } else {

                            this.weatherdata =
                                `Lämpötila on  ${res.main.temp}C, ` +
                                `ja ilmankosteus on ${res.main.humidity}%`;

                        }
                    },
                error: err => console.log('Tietoja ei löydy. Error code: %s, URL: %s', err.message, err.url)
            });
    }

    /*    ngOnInit(){
            this.searchInput.setValue("New York"); // Oletuksena haetaan säädataa Nyyjoorkista
        }
    */

    // getWeather palauttaa observablen joka emitoi sisältämiään alkioita streamina
    // Web-Apin data tulee siis observablesta. Tämä metodi voisi olla myös servicessä
    getWeather(city: string): Observable<any> {
        return this.http.get(this.baseWeatherURL + city + this.urlSuffix);
    }
}
